#include <string>

#include <glbase.hpp>
#include <globject.hpp>


struct GLObjectTest : protected GLObject<GL_TEXTURE>
{
    explicit GLObjectTest(std::string const & label)
    :
        GLObject(label)
    {}
};


int main()
{
    static_assert(
        sizeof(GLObjectTest) == sizeof(GLuint),
        "GLObject must be the same size as GLuint"
    );

    GLBase::debug(2);
    GLObjectTest("Test object");
}
